# Setup and Installation on Raspberry Pi
====================

## Setup the SD card
1) Setup a Raspbian OS image on the SD card
    - See https://www.raspberrypi.org/software/
        - Download Raspberry Pi Imager
        - Put in SD Card in PC
        - Open Raspberry Pi Imager, Select OS (Raspbian OS with desktop), and select the SD card

2) Create an empty ssh file on the root folder of the SD card
	- if on Mac: $ touch /Volumes/boot/ssh
	- (No need to add anything to this file. It should be empty.) 
  
3) Create a wpa_supplicant.conf file on the root folder of the SD card (for accessing the raspberry through wifi)
   - if on Mac: $ touch /Volumes/boot/wpa_supplicant.conf
   - enter the following into that file:
    ```bash
    country=NO
    ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev 
    update_config=1 
    
    network={ 
        ssid="NETWORK-NAME" 
        psk="NETWORK-PASSWORD" 
    }
    ```

## Setting up the Raspberry Pi

1) Enter the SD card into the Raspberry, and start it by connecting it to the power source. Then wait 10-15 seconds for it to start up.

2) Connect to it through SSH on your other computer:
	- $ ssh pi@raspberrypi.local
	- password: raspbian

3) change password (if you want):
	- $ passwd

4) Upgrade the Raspbian packages and setup locale environment
	- $ sudo apt-get update -y 
	- $ sudo apt-get upgrade -y 
	- $ sudo apt install raspberrypi-kernel raspberrypi-kernel-headers
	

5) Setup locale environment
    - $ sudo raspi-config
    	- -> localization options
        	- -> L1 configure language -> all -> en_GB.UTF-8
            - -> l2 configure tiemzone
    - $ sudo nano /etc/default/locale
     	- enter this into that file:
		```bash
        LC_CTYPE="en_GB.UTF-8"
		LC_ALL="en_GB.UTF-8"
        LANG="en_GB.UTF-8"
		LANGUAGE="en_GB.UTF-8"
        ```

6) Install Python 3
	- $ sudo apt update
	- $ sudo apt install python3
	- $ sudo pip3 install --upgrade pip

7) Install git, docker, and docker-compose
	- $ sudo apt-get install git
	- $ curl -sSL https://get.docker.com | sh
	- $ sudo pip3 install docker-compose

8) Enable the docker system service to start containers on boot, and enable use of docker without sudo
	- $ sudo systemctl enable docker
	- $ sudo groupadd docker
	- $ sudo usermod -aG docker $USER
	
9) Check if docker is installed:
	- $ docker


## Installing our own software

0) __First change user to root__ (there were some permission errors when running the software as the pi user). All the following commands in this section is done as root.
   - $ sudo su

1) Install and run the raspberry software (data collector and API, as of now)
	- $ cd /home/pi
	- $ git clone https://github.com/Master-thesis-Eric-Torn/raspberry-sw.git
	- $ cd raspberry-sw
	- $ docker-compose up --build -d

        This will start two __docker containers__, which runs the web-server and enables the API interface (getting data, starting/stopping the data collector, etc.). Because of the -d argument, the containers will run in the background.

2) Check if the three containers are running (ending in _nginx, _flask and _ngrok):
    - $ docker ps

3) Check that the webserver is running correctly:
	-  go to http://raspberrypi.local in your browser
      	-  Should show some text with how to use the api

__NOTE: only http:// works, not https://__


## Starting the data collector
(More info in the [README.md](../README.md) in the raspberry-sw folder)

1) Start up a terminal and enter the python interpreter (or open a python file)

2) When starting the data collector for the first time, you need to start it with some parameters. Then send a POST request to the API url.
    ```python
    # Note that Params are optional after the first start. It will use last-used params if none are supplied in the post request.
    params = { 
        'tibber': {
            'api_key': 'api-key',
            'sampling_time': 60, # [minutes] - Optional
            'rt_sampling_time': 20 # [seconds] - Optional
        },
        'sensibo': {
            'api_key': 'api-key',
            'sampling_time': 5, # [minutes] - Optional
        },
        'open_weather_map': {
            'api_key': 'api-key',
            'lat': "63.4",
            'lon': "10.335",
            'sampling_time': 60, # [minutes] - Optional
        },
        'backup': { # Optional
            'run_backups': True, # Optional (Defaults to True)
        },
    }

    import requests
    
    response = requests.post('http://raspberrypi.local/api/start_data_collector', json=params)
    # (Note that https does not work)

    print(response.text)
    ```

3) Access data through: (Note that https does not work)
    - Static files: 
      - http://raspberrypi.local/data/
      - http://raspberrypi.local/backups/ (backup runs once a day at midnight)
    - API call (JSON):
      - http://raspberrypi.local/api/get_data/MET/daily/all
        - Again, see the [README.md](../README.md) for more details

4) Print the data_collector terminal log to see whats going on in there:
    - $ docker logs raspberry-sw_flask_1
      - the log updates whenever you go to the http://raspberrypi.local url in your web browser

## Accessing the Pi from the internet
In order for others to access the data and interact with the pi, I implemented ngrok (http://ngrok.com).

1) Go to http://raspberrypi.local:4551 (Note that https does not work)
   - Here you can see the ngrok address where other people can access the raspberry pi's web interface (it will look something like: https://SOMETHING.ngrok.io)
   - Note: The ngrok channel will change if the raspberry or docker is restarted.
   - Please send the ngrok address to me (Eric)

- You can now access the pi and its api two ways. Both accepts requests made in a web browser or through a script.
  - http://raspberry.local (locally)
  - https://SOMETHING.ngrok.io (anyone through internet)

## Updating the raspberry with latest software
If there are some updates to the github repo later on, here is how you update the raspberry:
```bash
$ ssh pi@raspberrypi.local

pi@raspberrypi.local    $ sudo su
root@raspberrypi.local  $ cd /home/pi/raspberry-sw
root@raspberrypi.local  $ docker-compose down
root@raspberrypi.local  $ git pull
root@raspberrypi.local  $ docker-compose up
```
